"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeySpace = void 0;
const Table_1 = require("./Table");
const Middleware_1 = require("./Middleware");
class KeySpace {
    constructor(Table, config) {
        this.Table = Table;
        this.config = config;
        this.indexAttributeKeys = (index) => {
            const indexConfig = this.Table.config.indexes[index];
            return indexConfig.sort ? [indexConfig.hash.key, indexConfig.sort.key] : [indexConfig.hash.key];
        };
        this.assertAttributesAndKeys = _ => { };
        this.client = config.client || Table.client;
        this.dkClient = Table.dkClient;
        this.dkClient.setClient(config.client);
        this.dkClient.setDefaults({ ...this.dkClient.defaults, ...config.defaults });
        this.dkClient.setMiddleware((0, Middleware_1.appendMiddleware)(this.dkClient.middleware, config.middleware || []));
        this.dkClient.setLogger(config.logger);
        this.tableName = Table.config.name;
        this.indexValueHandlers = config.indexValueHandlers;
    }
    configure(config) {
        return new KeySpace(this.Table, config);
    }
    get indexes() {
        return Object.keys(this.indexValueHandlers).filter((index) => this.Table.indexes.includes(index));
    }
    get primaryIndex() {
        return this.Table.primaryIndex;
    }
    get secondaryIndexes() {
        return this.indexes.filter((index) => index !== this.primaryIndex);
    }
    get attributeKeys() {
        return this.indexes.flatMap(index => this.indexAttributeKeys(index));
    }
    indexAttributeValue(index, key, params) {
        return this.indexValueHandlers[index][key](params);
    }
    keyOf(params) {
        return this.indexKeyOf(Table_1.primaryIndex, params);
    }
    indexKeyOf(index, params) {
        const keys = this.indexAttributeKeys(index);
        return Object.fromEntries(keys.map(key => [key, this.indexAttributeValue(index, key, params)]));
    }
    indexKeysOf(item) {
        return this.indexes.reduce((currentIndexKeys, index) => ({
            ...currentIndexKeys,
            ...this.indexKeyOf(index, item)
        }), {});
    }
    withIndexKeys(item) {
        return { ...item, ...this.indexKeysOf(item) };
    }
    omitIndexKeys(item) {
        const clone = { ...item };
        this.attributeKeys.forEach(key => {
            if (key in clone)
                delete clone[key];
        });
        return clone;
    }
}
exports.KeySpace = KeySpace;
