"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dkOp = exports.DkOp = void 0;
const convertObjectToUpdateExpression_1 = require("./util/convertObjectToUpdateExpression");
const convertObjectToUpdateExpression_2 = require("./util/convertObjectToUpdateExpression");
class DkOp {
    constructor(createUpdateExpressionPart) {
        this.createUpdateExpressionPart = createUpdateExpressionPart;
    }
}
exports.DkOp = DkOp;
const attributePath = ({ alias, precedingKeys }) => `${precedingKeys && precedingKeys.length > 0
    ? `${precedingKeys.map(precedingKey => `#${precedingKey}`).join('.')}.`
    : ''}#${alias}`;
exports.dkOp = {
    Value: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            return {
                updateExpression: `${attributePath({ alias, precedingKeys })} = :${alias}, `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    MapValue: (object) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const mappedObject = Object.fromEntries(Object.entries(object).map(([key, value]) => [key, exports.dkOp.Value(value)]));
            const updateExpressionPrecedingKeyPart = {
                updateExpression: '',
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {}
            };
            const updateExpressionParts = (0, convertObjectToUpdateExpression_2.createUpdateExpressionParts)(mappedObject, [...(precedingKeys || []), alias]);
            return (0, convertObjectToUpdateExpression_1.mergeUpdateExpressionParts)([updateExpressionPrecedingKeyPart, ...updateExpressionParts]);
        });
        return output;
    },
    Add: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = ${path} + :${alias}, `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    Minus: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = ${path} - :${alias}, `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    ListAppend: (value, end = 'tail') => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = list_append(${end === 'head' ? `:${alias}, ${path}` : `${path}, :${alias}`}), `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    IfNotExists: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = if_not_exists(${path}, :${alias}), `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    }
};
