"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamoDBKeySpaceStack = void 0;
const aws_cdk_lib_1 = require("aws-cdk-lib");
const aws_dynamodb_1 = require("aws-cdk-lib/aws-dynamodb");
class DynamoDBKeySpaceStack extends aws_cdk_lib_1.Stack {
    constructor(scope, id, props) {
        super(scope, id, props);
        const database = new aws_dynamodb_1.Table(this, 'database', {
            tableName: `${props.deploymentName}-database`,
            billingMode: aws_dynamodb_1.BillingMode.PAY_PER_REQUEST,
            removalPolicy: aws_cdk_lib_1.RemovalPolicy.DESTROY,
            partitionKey: { name: 'pk', type: aws_dynamodb_1.AttributeType.STRING },
            sortKey: { name: 'sk', type: aws_dynamodb_1.AttributeType.STRING }
        });
        database.addGlobalSecondaryIndex({
            indexName: `gsi0`,
            partitionKey: { name: `gsi0Pk`, type: aws_dynamodb_1.AttributeType.STRING },
            sortKey: { name: `gsi0Sk`, type: aws_dynamodb_1.AttributeType.STRING },
            projectionType: aws_dynamodb_1.ProjectionType.ALL
        });
        database.addGlobalSecondaryIndex({
            indexName: `gsi1`,
            partitionKey: { name: `gsi1Pk`, type: aws_dynamodb_1.AttributeType.NUMBER },
            sortKey: { name: `gsi1Sk`, type: aws_dynamodb_1.AttributeType.NUMBER },
            projectionType: aws_dynamodb_1.ProjectionType.ALL
        });
        database.addGlobalSecondaryIndex({
            indexName: `gsi2`,
            partitionKey: { name: `gsi2Pk`, type: aws_dynamodb_1.AttributeType.STRING },
            sortKey: { name: `gsi2Sk`, type: aws_dynamodb_1.AttributeType.NUMBER },
            projectionType: aws_dynamodb_1.ProjectionType.ALL
        });
        database.addGlobalSecondaryIndex({
            indexName: `gsi3`,
            partitionKey: { name: `gsi3Pk`, type: aws_dynamodb_1.AttributeType.NUMBER },
            sortKey: { name: `gsi3Sk`, type: aws_dynamodb_1.AttributeType.STRING },
            projectionType: aws_dynamodb_1.ProjectionType.ALL
        });
        database.addGlobalSecondaryIndex({
            indexName: `gsi4`,
            partitionKey: { name: `gsi4Pk`, type: aws_dynamodb_1.AttributeType.STRING },
            projectionType: aws_dynamodb_1.ProjectionType.ALL
        });
        database.addGlobalSecondaryIndex({
            indexName: `gsi5`,
            partitionKey: { name: `gsi5Pk`, type: aws_dynamodb_1.AttributeType.NUMBER },
            projectionType: aws_dynamodb_1.ProjectionType.ALL
        });
        this.tableName = new aws_cdk_lib_1.CfnOutput(this, `${props.deploymentName}-tableName`, {
            value: database.tableName,
            exportName: `${props.deploymentName}-tableName`
        });
        const scanDatabase = new aws_dynamodb_1.Table(this, 'scanDatabase', {
            tableName: `${props.deploymentName}-scan-database`,
            billingMode: aws_dynamodb_1.BillingMode.PAY_PER_REQUEST,
            removalPolicy: aws_cdk_lib_1.RemovalPolicy.DESTROY,
            partitionKey: { name: 'pk', type: aws_dynamodb_1.AttributeType.STRING },
            sortKey: { name: 'sk', type: aws_dynamodb_1.AttributeType.STRING }
        });
        this.scanTableName = new aws_cdk_lib_1.CfnOutput(this, `${props.deploymentName}-scanTableName`, {
            value: scanDatabase.tableName,
            exportName: `${props.deploymentName}-scanTableName`
        });
        const resetDatabase = new aws_dynamodb_1.Table(this, 'resetDatabase', {
            tableName: `${props.deploymentName}-reset-database`,
            billingMode: aws_dynamodb_1.BillingMode.PAY_PER_REQUEST,
            removalPolicy: aws_cdk_lib_1.RemovalPolicy.DESTROY,
            partitionKey: { name: 'pk', type: aws_dynamodb_1.AttributeType.STRING },
            sortKey: { name: 'sk', type: aws_dynamodb_1.AttributeType.STRING }
        });
        this.resetTableName = new aws_cdk_lib_1.CfnOutput(this, `${props.deploymentName}-resetTableName`, {
            value: resetDatabase.tableName,
            exportName: `${props.deploymentName}-resetTableName`
        });
    }
}
exports.DynamoDBKeySpaceStack = DynamoDBKeySpaceStack;
