"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DkBatchGetCommand = void 0;
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const Command_1 = require("./Command");
const keyCapitalize_1 = require("../util/keyCapitalize");
const defaults_1 = require("../util/defaults");
const Middleware_1 = require("../Middleware");
const BATCH_GET_COMMAND_INPUT_DATA_TYPE = 'BatchGetCommandInput';
const BATCH_GET_COMMAND_INPUT_HOOK = ['CommandInput', 'ReadCommandInput', BATCH_GET_COMMAND_INPUT_DATA_TYPE];
const BATCH_GET_COMMAND_OUTPUT_DATA_TYPE = 'BatchGetCommandOutput';
const BATCH_GET_COMMAND_OUTPUT_HOOK = [
    'CommandOutput',
    'ReadCommandOutput',
    BATCH_GET_COMMAND_OUTPUT_DATA_TYPE
];
class DkBatchGetCommand extends Command_1.DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = { dataType: BATCH_GET_COMMAND_INPUT_DATA_TYPE, hooks: BATCH_GET_COMMAND_INPUT_HOOK };
        this.outputMiddlewareConfig = { dataType: BATCH_GET_COMMAND_OUTPUT_DATA_TYPE, hooks: BATCH_GET_COMMAND_OUTPUT_HOOK };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = (0, defaults_1.applyDefaults)(this.input, defaults, ['returnConsumedCapacity']);
            const { data: postMiddlewareInput } = await (0, Middleware_1.executeMiddlewares)([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: postDefaultsInput
            }, middleware);
            const { requests, ...rest } = postMiddlewareInput;
            const formattedInput = {
                requestItems: Object.fromEntries(Object.entries(requests).map(([tableName, keysAndAttributes]) => [tableName, (0, keyCapitalize_1.upperCaseKeys)(keysAndAttributes)])),
                ...rest
            };
            const upperCaseInput = (0, keyCapitalize_1.upperCaseKeys)(formattedInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = (0, keyCapitalize_1.lowerCaseKeys)(output);
            const { responses, unprocessedKeys, ...rest } = lowerCaseOutput;
            const items = Object.fromEntries(Object.entries(responses || {}).map(([tableName, tableItems]) => {
                const typedTableItems = (tableItems || []);
                return [tableName, typedTableItems];
            }));
            const formattedUnprocessedRequests = Object.fromEntries(Object.entries(unprocessedKeys || {}).map(([tableName, request]) => {
                if (!request)
                    return [tableName, undefined];
                const { Keys, ...unprocessedRest } = request;
                const typedKeys = (Keys || []);
                return [tableName, (0, keyCapitalize_1.lowerCaseKeys)({ Keys: typedKeys, ...unprocessedRest })];
            }));
            const formattedOutput = {
                ...rest,
                items,
                unprocessedRequests: formattedUnprocessedRequests
            };
            const { data: postMiddlewareOutput } = await (0, Middleware_1.executeMiddlewares)([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: formattedOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity) {
                for (const consumedCapacity of postMiddlewareOutput.consumedCapacity) {
                    await (0, Middleware_1.executeMiddleware)('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: consumedCapacity }, middleware);
                }
            }
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new lib_dynamodb_1.BatchGetCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
exports.DkBatchGetCommand = DkBatchGetCommand;
