"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DkBatchWriteCommand = void 0;
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const Command_1 = require("./Command");
const keyCapitalize_1 = require("../util/keyCapitalize");
const defaults_1 = require("../util/defaults");
const Middleware_1 = require("../Middleware");
const BATCH_WRITE_COMMAND_INPUT_DATA_TYPE = 'BatchWriteCommandInput';
const BATCH_WRITE_COMMAND_INPUT_HOOK = [
    'CommandInput',
    'WriteCommandInput',
    BATCH_WRITE_COMMAND_INPUT_DATA_TYPE
];
const BATCH_WRITE_COMMAND_OUTPUT_DATA_TYPE = 'BatchWriteCommandOutput';
const BATCH_WRITE_COMMAND_OUTPUT_HOOK = [
    'CommandOutput',
    'WriteCommandOutput',
    BATCH_WRITE_COMMAND_OUTPUT_DATA_TYPE
];
class DkBatchWriteCommand extends Command_1.DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = { dataType: BATCH_WRITE_COMMAND_INPUT_DATA_TYPE, hooks: BATCH_WRITE_COMMAND_INPUT_HOOK };
        this.outputMiddlewareConfig = { dataType: BATCH_WRITE_COMMAND_OUTPUT_DATA_TYPE, hooks: BATCH_WRITE_COMMAND_OUTPUT_HOOK };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = (0, defaults_1.applyDefaults)(this.input, defaults, [
                'returnConsumedCapacity',
                'returnItemCollectionMetrics'
            ]);
            const { data: postMiddlewareInput } = await (0, Middleware_1.executeMiddlewares)([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: postDefaultsInput
            }, middleware);
            const { requests, ...rest } = postMiddlewareInput;
            const formattedInput = {
                requestItems: Object.fromEntries(Object.entries(requests).map(([tableName, requests]) => {
                    const formattedRequests = requests.map(request => {
                        if ('put' in request) {
                            return {
                                PutRequest: {
                                    Item: request.put
                                }
                            };
                        }
                        return {
                            DeleteRequest: {
                                Key: request.delete
                            }
                        };
                    });
                    return [tableName, formattedRequests];
                })),
                ...rest
            };
            const upperCaseInput = (0, keyCapitalize_1.upperCaseKeys)(formattedInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = (0, keyCapitalize_1.lowerCaseKeys)(output);
            const { unprocessedItems, ...rest } = lowerCaseOutput;
            const formattedUnprocessedRequests = Object.fromEntries(Object.entries(unprocessedItems || {}).map(([tableName, requests]) => {
                const formattedRequests = requests.map(request => {
                    var _a;
                    if (request.PutRequest) {
                        const item = request.PutRequest.Item;
                        return {
                            put: item
                        };
                    }
                    const key = (_a = request.DeleteRequest) === null || _a === void 0 ? void 0 : _a.Key;
                    return {
                        delete: key
                    };
                });
                return [tableName, formattedRequests];
            }));
            const formattedOutput = {
                ...rest,
                unprocessedRequests: formattedUnprocessedRequests
            };
            const { data: postMiddlewareOutput } = await (0, Middleware_1.executeMiddlewares)([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: formattedOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity) {
                for (const consumedCapacity of postMiddlewareOutput.consumedCapacity) {
                    await (0, Middleware_1.executeMiddleware)('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: consumedCapacity }, middleware);
                }
            }
            if (postMiddlewareOutput.itemCollectionMetrics) {
                await (0, Middleware_1.executeMiddleware)('ItemCollectionMetrics', { dataType: 'ItemCollectionMetrics', data: postMiddlewareOutput.itemCollectionMetrics }, middleware);
            }
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new lib_dynamodb_1.BatchWriteCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
exports.DkBatchWriteCommand = DkBatchWriteCommand;
