"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DkDeleteCommand = void 0;
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const Command_1 = require("./Command");
const keyCapitalize_1 = require("../util/keyCapitalize");
const defaults_1 = require("../util/defaults");
const Middleware_1 = require("../Middleware");
const returnValuesAttributes_1 = require("../util/returnValuesAttributes");
const DELETE_COMMAND_INPUT_DATA_TYPE = 'DeleteCommandInput';
const DELETE_COMMAND_INPUT_HOOK = ['CommandInput', 'WriteCommandInput', DELETE_COMMAND_INPUT_DATA_TYPE];
const DELETE_COMMAND_OUTPUT_DATA_TYPE = 'DeleteCommandOutput';
const DELETE_COMMAND_OUTPUT_HOOK = ['CommandOutput', 'WriteCommandOutput', DELETE_COMMAND_OUTPUT_DATA_TYPE];
class DkDeleteCommand extends Command_1.DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = { dataType: DELETE_COMMAND_INPUT_DATA_TYPE, hooks: DELETE_COMMAND_INPUT_HOOK };
        this.outputMiddlewareConfig = { dataType: DELETE_COMMAND_OUTPUT_DATA_TYPE, hooks: DELETE_COMMAND_OUTPUT_HOOK };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = (0, defaults_1.applyDefaults)(this.input, defaults, [
                'returnConsumedCapacity',
                'returnItemCollectionMetrics',
                'returnValuesOnConditionCheckFailure'
            ]);
            const formattedInput = {
                ...postDefaultsInput,
                returnValues: postDefaultsInput.returnValues || undefined
            };
            const { data: postMiddlewareInput } = await (0, Middleware_1.executeMiddlewares)([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: formattedInput
            }, middleware);
            const upperCaseInput = (0, keyCapitalize_1.upperCaseKeys)(postMiddlewareInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = (0, keyCapitalize_1.lowerCaseKeys)(output);
            const attributes = output.Attributes;
            (0, returnValuesAttributes_1.assertReturnValuesAttributes)(attributes, this.input.returnValues);
            const formattedOutput = {
                ...lowerCaseOutput,
                attributes
            };
            const { data: postMiddlewareOutput } = await (0, Middleware_1.executeMiddlewares)([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: formattedOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity)
                await (0, Middleware_1.executeMiddleware)('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: postMiddlewareOutput.consumedCapacity }, middleware);
            if (postMiddlewareOutput.itemCollectionMetrics)
                await (0, Middleware_1.executeMiddleware)('ItemCollectionMetrics', { dataType: 'ItemCollectionMetrics', data: postMiddlewareOutput.itemCollectionMetrics }, middleware);
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new lib_dynamodb_1.DeleteCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
exports.DkDeleteCommand = DkDeleteCommand;
