"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DkGetCommand = void 0;
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const Command_1 = require("./Command");
const keyCapitalize_1 = require("../util/keyCapitalize");
const defaults_1 = require("../util/defaults");
const Middleware_1 = require("../Middleware");
const GET_COMMAND_INPUT_DATA_TYPE = 'GetCommandInput';
const GET_COMMAND_INPUT_HOOK = ['CommandInput', 'ReadCommandInput', GET_COMMAND_INPUT_DATA_TYPE];
const GET_COMMAND_OUTPUT_DATA_TYPE = 'GetCommandOutput';
const GET_COMMAND_OUTPUT_HOOK = ['CommandOutput', 'ReadCommandOutput', GET_COMMAND_OUTPUT_DATA_TYPE];
class DkGetCommand extends Command_1.DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = { dataType: GET_COMMAND_INPUT_DATA_TYPE, hooks: GET_COMMAND_INPUT_HOOK };
        this.outputMiddlewareConfig = { dataType: GET_COMMAND_OUTPUT_DATA_TYPE, hooks: GET_COMMAND_OUTPUT_HOOK };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = (0, defaults_1.applyDefaults)(this.input, defaults, ['returnConsumedCapacity']);
            const { data: postMiddlewareInput } = await (0, Middleware_1.executeMiddlewares)([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: postDefaultsInput
            }, middleware);
            const upperCaseInput = (0, keyCapitalize_1.upperCaseKeys)(postMiddlewareInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = (0, keyCapitalize_1.lowerCaseKeys)(output);
            const item = output.Item;
            if (!item)
                throw new Error('Item Not Found');
            const formattedOutput = {
                ...lowerCaseOutput,
                item
            };
            const { data: postMiddlewareOutput } = await (0, Middleware_1.executeMiddlewares)([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: formattedOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity)
                await (0, Middleware_1.executeMiddleware)('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: postMiddlewareOutput.consumedCapacity }, middleware);
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new lib_dynamodb_1.GetCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
exports.DkGetCommand = DkGetCommand;
