"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DkQueryCommand = exports.DkQueryItemsSort = void 0;
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const Command_1 = require("./Command");
const keyCapitalize_1 = require("../util/keyCapitalize");
const defaults_1 = require("../util/defaults");
const Middleware_1 = require("../Middleware");
var DkQueryItemsSort;
(function (DkQueryItemsSort) {
    DkQueryItemsSort["ASCENDING"] = "ascending";
    DkQueryItemsSort["DESCENDING"] = "descending";
})(DkQueryItemsSort || (exports.DkQueryItemsSort = DkQueryItemsSort = {}));
const QUERY_COMMAND_INPUT_DATA_TYPE = 'QueryCommandInput';
const QUERY_COMMAND_INPUT_HOOK = ['CommandInput', 'ReadCommandInput', QUERY_COMMAND_INPUT_DATA_TYPE];
const QUERY_COMMAND_OUTPUT_DATA_TYPE = 'QueryCommandOutput';
const QUERY_COMMAND_OUTPUT_HOOK = ['CommandOutput', 'ReadCommandOutput', QUERY_COMMAND_OUTPUT_DATA_TYPE];
class DkQueryCommand extends Command_1.DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = { dataType: QUERY_COMMAND_INPUT_DATA_TYPE, hooks: QUERY_COMMAND_INPUT_HOOK };
        this.outputMiddlewareConfig = { dataType: QUERY_COMMAND_OUTPUT_DATA_TYPE, hooks: QUERY_COMMAND_OUTPUT_HOOK };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = (0, defaults_1.applyDefaults)(this.input, defaults, ['returnConsumedCapacity']);
            const { data: postMiddlewareInput } = await (0, Middleware_1.executeMiddlewares)([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: postDefaultsInput
            }, middleware);
            const { cursorKey, index, sort = DkQueryItemsSort.ASCENDING, ...rest } = postMiddlewareInput;
            const formattedInput = {
                exclusiveStartKey: cursorKey,
                indexName: index,
                scanIndexForward: sort === DkQueryItemsSort.ASCENDING ? true : false,
                ...rest
            };
            const upperCaseInput = (0, keyCapitalize_1.upperCaseKeys)(formattedInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = (0, keyCapitalize_1.lowerCaseKeys)(output);
            const items = (output.Items || []);
            const cursorKey = output.LastEvaluatedKey;
            const formattedOutput = {
                ...lowerCaseOutput,
                items,
                cursorKey
            };
            const { data: postMiddlewareOutput } = await (0, Middleware_1.executeMiddlewares)([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: formattedOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity)
                await (0, Middleware_1.executeMiddleware)('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: postMiddlewareOutput.consumedCapacity }, middleware);
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new lib_dynamodb_1.QueryCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
exports.DkQueryCommand = DkQueryCommand;
