"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DkTransactGetCommand = void 0;
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const Command_1 = require("./Command");
const keyCapitalize_1 = require("../util/keyCapitalize");
const defaults_1 = require("../util/defaults");
const Middleware_1 = require("../Middleware");
const TRANSACT_GET_COMMAND_INPUT_DATA_TYPE = 'TransactGetCommandInput';
const TRANSACT_GET_COMMAND_INPUT_HOOK = [
    'CommandInput',
    'ReadCommandInput',
    TRANSACT_GET_COMMAND_INPUT_DATA_TYPE
];
const TRANSACT_GET_COMMAND_OUTPUT_DATA_TYPE = 'TransactGetCommandOutput';
const TRANSACT_GET_COMMAND_OUTPUT_HOOK = [
    'CommandOutput',
    'ReadCommandOutput',
    TRANSACT_GET_COMMAND_OUTPUT_DATA_TYPE
];
class DkTransactGetCommand extends Command_1.DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = { dataType: TRANSACT_GET_COMMAND_INPUT_DATA_TYPE, hooks: TRANSACT_GET_COMMAND_INPUT_HOOK };
        this.outputMiddlewareConfig = { dataType: TRANSACT_GET_COMMAND_OUTPUT_DATA_TYPE, hooks: TRANSACT_GET_COMMAND_OUTPUT_HOOK };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = (0, defaults_1.applyDefaults)(this.input, defaults, ['returnConsumedCapacity']);
            const { data: postMiddlewareInput } = await (0, Middleware_1.executeMiddlewares)([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: postDefaultsInput
            }, middleware);
            const { requests, ...rest } = postMiddlewareInput;
            const formattedInput = {
                transactItems: requests.map(request => ({ Get: (0, keyCapitalize_1.upperCaseKeys)(request) })),
                ...rest
            };
            const upperCaseInput = (0, keyCapitalize_1.upperCaseKeys)(formattedInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = (0, keyCapitalize_1.lowerCaseKeys)(output);
            const { responses, ...rest } = lowerCaseOutput;
            const items = (responses === null || responses === void 0 ? void 0 : responses.map(response => {
                return response.Item;
            }).filter((item) => !!item)) || [];
            const formattedOutput = {
                ...rest,
                items
            };
            const { data: postMiddlewareOutput } = await (0, Middleware_1.executeMiddlewares)([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: formattedOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity) {
                for (const consumedCapacity of postMiddlewareOutput.consumedCapacity) {
                    await (0, Middleware_1.executeMiddleware)('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: consumedCapacity }, middleware);
                }
            }
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new lib_dynamodb_1.TransactGetCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
exports.DkTransactGetCommand = DkTransactGetCommand;
