"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DkTransactWriteCommand = void 0;
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const Command_1 = require("./Command");
const keyCapitalize_1 = require("../util/keyCapitalize");
const defaults_1 = require("../util/defaults");
const Middleware_1 = require("../Middleware");
const TRANSACT_WRITE_COMMAND_INPUT_DATA_TYPE = 'TransactWriteCommandInput';
const TRANSACT_WRITE_COMMAND_INPUT_HOOK = [
    'CommandInput',
    'WriteCommandInput',
    TRANSACT_WRITE_COMMAND_INPUT_DATA_TYPE
];
const TRANSACT_WRITE_COMMAND_OUTPUT_DATA_TYPE = 'TransactWriteCommandOutput';
const TRANSACT_WRITE_COMMAND_OUTPUT_HOOK = [
    'CommandOutput',
    'WriteCommandOutput',
    TRANSACT_WRITE_COMMAND_OUTPUT_DATA_TYPE
];
class DkTransactWriteCommand extends Command_1.DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = {
            dataType: TRANSACT_WRITE_COMMAND_INPUT_DATA_TYPE,
            hooks: TRANSACT_WRITE_COMMAND_INPUT_HOOK
        };
        this.outputMiddlewareConfig = {
            dataType: TRANSACT_WRITE_COMMAND_OUTPUT_DATA_TYPE,
            hooks: TRANSACT_WRITE_COMMAND_OUTPUT_HOOK
        };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = (0, defaults_1.applyDefaults)(this.input, defaults, [
                'returnConsumedCapacity',
                'returnItemCollectionMetrics'
            ]);
            const { data: postMiddlewareInput } = await (0, Middleware_1.executeMiddlewares)([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: postDefaultsInput
            }, middleware);
            const { requests, ...rest } = postMiddlewareInput;
            const formattedInput = {
                transactItems: requests.map(request => {
                    if (request.type === 'conditionCheck') {
                        return {
                            ConditionCheck: (0, keyCapitalize_1.upperCaseKeys)(request)
                        };
                    }
                    if (request.type === 'put') {
                        return {
                            Put: (0, keyCapitalize_1.upperCaseKeys)(request)
                        };
                    }
                    if (request.type === 'delete') {
                        return {
                            Delete: (0, keyCapitalize_1.upperCaseKeys)(request)
                        };
                    }
                    if (request.type === 'update') {
                        return {
                            Update: (0, keyCapitalize_1.upperCaseKeys)(request)
                        };
                    }
                    return {};
                }),
                ...rest
            };
            const upperCaseInput = (0, keyCapitalize_1.upperCaseKeys)(formattedInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = (0, keyCapitalize_1.lowerCaseKeys)(output);
            const { data: postMiddlewareOutput } = await (0, Middleware_1.executeMiddlewares)([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: lowerCaseOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity) {
                for (const consumedCapacity of postMiddlewareOutput.consumedCapacity) {
                    await (0, Middleware_1.executeMiddleware)('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: consumedCapacity }, middleware);
                }
            }
            if (postMiddlewareOutput.itemCollectionMetrics) {
                await (0, Middleware_1.executeMiddleware)('ItemCollectionMetrics', { dataType: 'ItemCollectionMetrics', data: postMiddlewareOutput.itemCollectionMetrics }, middleware);
            }
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new lib_dynamodb_1.TransactWriteCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
exports.DkTransactWriteCommand = DkTransactWriteCommand;
