"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.batchGetItems = exports.batchGetTableItems = void 0;
const BatchGet_1 = require("../command/BatchGet");
const batchGetTableItems = async (Table, keys, input, dkClient = Table.dkClient) => {
    const { pageLimit = 100, returnConsumedCapacity, ...rest } = input || {};
    if (keys.length === 0)
        return {
            items: [],
            unprocessedRequests: {
                keys: [],
                ...rest
            }
        };
    const limitedPageLimit = Math.min(pageLimit, 100);
    const recurse = async (remainingKeys) => {
        const currentKeys = remainingKeys.slice(0, limitedPageLimit);
        const output = await dkClient.send(new BatchGet_1.DkBatchGetCommand({
            requests: {
                [Table.tableName]: {
                    keys: currentKeys,
                    ...rest
                }
            },
            returnConsumedCapacity
        }));
        const nextRemainingKeys = remainingKeys.slice(limitedPageLimit);
        const items = output.items[Table.tableName];
        const unprocessedRequests = output.unprocessedRequests[Table.tableName] || { keys: [] };
        if (nextRemainingKeys.length === 0) {
            return {
                items,
                unprocessedRequests
            };
        }
        const nextPage = await recurse(nextRemainingKeys);
        return {
            items: [...items, ...nextPage.items],
            unprocessedRequests: {
                keys: [...unprocessedRequests.keys, ...nextPage.unprocessedRequests.keys],
                ...rest
            }
        };
    };
    return recurse(keys);
};
exports.batchGetTableItems = batchGetTableItems;
const batchGetItems = async (KeySpace, keys, input) => {
    const output = await (0, exports.batchGetTableItems)(KeySpace.Table, keys.map(key => KeySpace.keyOf(key), input), input, KeySpace.dkClient);
    return {
        ...output,
        items: output.items.map(item => {
            KeySpace.assertAttributesAndKeys(item);
            return KeySpace.omitIndexKeys(item);
        })
    };
};
exports.batchGetItems = batchGetItems;
