"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.batchWriteItems = exports.batchWriteTableItems = void 0;
const BatchWrite_1 = require("../command/BatchWrite");
const batchWriteTableItems = async (Table, requests, input, dkClient = Table.dkClient) => {
    if (requests.length === 0) {
        return {
            unprocessedRequests: []
        };
    }
    const pageLimit = (input === null || input === void 0 ? void 0 : input.pageLimit) ? Math.min(input.pageLimit, 25) : 25;
    const recurse = async (remainingRequests) => {
        const currentRequests = remainingRequests.slice(0, pageLimit);
        const output = await dkClient.send(new BatchWrite_1.DkBatchWriteCommand({
            ...input,
            requests: {
                [Table.tableName]: currentRequests
            }
        }));
        const unprocessedRequests = output.unprocessedRequests[Table.tableName] || [];
        const nextRemainingRequests = remainingRequests.slice(pageLimit);
        if (nextRemainingRequests.length === 0) {
            return {
                ...output,
                unprocessedRequests: unprocessedRequests
            };
        }
        const nextPage = await recurse(nextRemainingRequests);
        return {
            ...output,
            unprocessedRequests: [...unprocessedRequests, ...nextPage.unprocessedRequests]
        };
    };
    return recurse(requests);
};
exports.batchWriteTableItems = batchWriteTableItems;
const batchWriteItems = async (KeySpace, requests, input) => (0, exports.batchWriteTableItems)(KeySpace.Table, requests.map(request => {
    if ('put' in request) {
        return { put: KeySpace.withIndexKeys(request.put) };
    }
    return { delete: KeySpace.keyOf(request.delete) };
}), input, KeySpace.dkClient);
exports.batchWriteItems = batchWriteItems;
