"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryQuickItems = exports.queryQuickTableItems = void 0;
const Table_1 = require("../Table");
const query_1 = require("./query");
const createSortKeyQuery_1 = require("../util/createSortKeyQuery");
const queryQuickTableItems = async (Table, input, dkClient = Table.dkClient) => {
    var _a;
    const index = input.index || Table_1.primaryIndex;
    const { hashKeyParams, beginsWith, greaterThan, lessThan, ...inputRest } = input;
    const hashKey = Table.config.indexes[index].hash.key;
    const hashValue = hashKeyParams[hashKey];
    const sortKey = (_a = Table.config.indexes[index].sort) === null || _a === void 0 ? void 0 : _a.key;
    const sortParams = (0, createSortKeyQuery_1.createQueryQuickSort)(sortKey, { beginsWith, greaterThan, lessThan });
    const output = await (0, query_1.queryTableItems)(Table, {
        ...inputRest,
        keyConditionExpression: `${hashKey} = :hashValue ${sortParams.keyConditionExpression || ''}`,
        expressionAttributeValues: {
            [`:hashValue`]: hashValue,
            ...sortParams.expressionAttributeValues,
            ...input.expressionAttributeValues
        }
    }, dkClient);
    return output;
};
exports.queryQuickTableItems = queryQuickTableItems;
const queryQuickItems = async (KeySpace, input) => {
    const index = input.index || Table_1.primaryIndex;
    const hashKeyValue = KeySpace.indexAttributeValue(index, KeySpace.Table.config.indexes[index].hash.key, input.hashKeyParams);
    const hashKeyParams = {
        [KeySpace.Table.config.indexes[index].hash.key]: hashKeyValue
    };
    const output = await (0, exports.queryQuickTableItems)(KeySpace.Table, { ...input, hashKeyParams }, KeySpace.dkClient);
    return {
        ...output,
        items: output.items.map(item => {
            KeySpace.assertAttributesAndKeys(item);
            return KeySpace.omitIndexKeys(item);
        })
    };
};
exports.queryQuickItems = queryQuickItems;
