"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.scanTableItems = void 0;
const Scan_1 = require("../command/Scan");
const scanTableItems = async (Table, input) => {
    const recurse = async (totalCount, pageCursorKey) => {
        const { autoPage, pageLimit, totalLimit, ...inputRest } = input || {};
        const output = await Table.dkClient.send(new Scan_1.DkScanCommand({
            ...inputRest,
            tableName: Table.tableName,
            cursorKey: pageCursorKey,
            limit: pageLimit
        }));
        const { items, cursorKey, count = 0, ...rest } = output;
        const newTotalCount = totalCount + items.length;
        if (!autoPage || !cursorKey || (totalLimit && newTotalCount >= totalLimit)) {
            return {
                items: items.slice(0, totalLimit),
                cursorKey,
                count,
                ...rest
            };
        }
        const nextPage = await recurse(newTotalCount, cursorKey);
        return {
            items: [...items, ...nextPage.items].slice(0, totalLimit),
            cursorKey: nextPage.cursorKey,
            count: count + (nextPage.count || 0),
            ...rest
        };
    };
    return recurse(0, input === null || input === void 0 ? void 0 : input.cursorKey);
};
exports.scanTableItems = scanTableItems;
