"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertObjectToUpdateExpression = exports.mergeUpdateExpressionParts = exports.createUpdateExpressionParts = void 0;
const UpdateOp_1 = require("../UpdateOp");
const utils_1 = require("./utils");
const createUpdateExpressionPart = (alias, key, value, precedingKeys) => ({
    updateExpression: `${precedingKeys ? `${precedingKeys.map(key => `#${key}`).join('.')}.` : ''}#${alias} = :${alias}, `,
    expressionAttributeNames: {
        [`#${alias}`]: key
    },
    expressionAttributeValues: {
        [`:${alias}`]: value
    }
});
const createUpdateExpressionParts = (attributes, precedingKeys) => {
    return Object.entries(attributes).flatMap(entry => {
        const [key, value] = entry;
        const alias = (0, utils_1.randomString)(10);
        if (value instanceof UpdateOp_1.DkOp) {
            const updateExpressionPart = value.createUpdateExpressionPart({ key, alias, precedingKeys });
            return [updateExpressionPart];
        }
        if (value !== null && typeof value === 'object' && !Array.isArray(value)) {
            const updateExpressionPrecedingKeyPart = {
                updateExpression: '',
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {}
            };
            return [
                updateExpressionPrecedingKeyPart,
                ...(0, exports.createUpdateExpressionParts)(value, precedingKeys ? [...precedingKeys, alias] : [alias])
            ];
        }
        const updateExpressionPart = createUpdateExpressionPart(alias, key, value, precedingKeys);
        return [updateExpressionPart];
    });
};
exports.createUpdateExpressionParts = createUpdateExpressionParts;
const mergeUpdateExpressionParts = (parts) => parts.reduce((accumulator, expressionPart) => {
    return {
        updateExpression: (accumulator.updateExpression || '') + expressionPart.updateExpression,
        expressionAttributeNames: {
            ...accumulator.expressionAttributeNames,
            ...expressionPart.expressionAttributeNames
        },
        expressionAttributeValues: {
            ...accumulator.expressionAttributeValues,
            ...expressionPart.expressionAttributeValues
        }
    };
}, {
    updateExpression: '',
    expressionAttributeNames: {},
    expressionAttributeValues: {}
});
exports.mergeUpdateExpressionParts = mergeUpdateExpressionParts;
const convertObjectToUpdateExpression = (attributes) => {
    const parts = (0, exports.createUpdateExpressionParts)(attributes);
    if (parts.length === 0)
        throw new Error('Invalid update object');
    const merged = (0, exports.mergeUpdateExpressionParts)(parts);
    const updateExpression = {
        ...merged,
        updateExpression: `SET ${merged.updateExpression.trim().slice(0, -1)}`
    };
    return updateExpression;
};
exports.convertObjectToUpdateExpression = convertObjectToUpdateExpression;
