import { mergeUpdateExpressionParts } from './util/convertObjectToUpdateExpression';
import { createUpdateExpressionParts } from './util/convertObjectToUpdateExpression';
export class DkOp {
    constructor(createUpdateExpressionPart) {
        this.createUpdateExpressionPart = createUpdateExpressionPart;
    }
}
const attributePath = ({ alias, precedingKeys }) => `${precedingKeys && precedingKeys.length > 0
    ? `${precedingKeys.map(precedingKey => `#${precedingKey}`).join('.')}.`
    : ''}#${alias}`;
export const dkOp = {
    Value: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            return {
                updateExpression: `${attributePath({ alias, precedingKeys })} = :${alias}, `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    MapValue: (object) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const mappedObject = Object.fromEntries(Object.entries(object).map(([key, value]) => [key, dkOp.Value(value)]));
            const updateExpressionPrecedingKeyPart = {
                updateExpression: '',
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {}
            };
            const updateExpressionParts = createUpdateExpressionParts(mappedObject, [...(precedingKeys || []), alias]);
            return mergeUpdateExpressionParts([updateExpressionPrecedingKeyPart, ...updateExpressionParts]);
        });
        return output;
    },
    Add: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = ${path} + :${alias}, `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    Minus: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = ${path} - :${alias}, `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    ListAppend: (value, end = 'tail') => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = list_append(${end === 'head' ? `:${alias}, ${path}` : `${path}, :${alias}`}), `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    IfNotExists: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = if_not_exists(${path}, :${alias}), `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    }
};
