import { TransactGetCommand } from '@aws-sdk/lib-dynamodb';
import { DkCommand } from './Command';
import { lowerCaseKeys, upperCaseKeys } from '../util/keyCapitalize';
import { applyDefaults } from '../util/defaults';
import { executeMiddlewares, executeMiddleware } from '../Middleware';
const TRANSACT_GET_COMMAND_INPUT_DATA_TYPE = 'TransactGetCommandInput';
const TRANSACT_GET_COMMAND_INPUT_HOOK = [
    'CommandInput',
    'ReadCommandInput',
    TRANSACT_GET_COMMAND_INPUT_DATA_TYPE
];
const TRANSACT_GET_COMMAND_OUTPUT_DATA_TYPE = 'TransactGetCommandOutput';
const TRANSACT_GET_COMMAND_OUTPUT_HOOK = [
    'CommandOutput',
    'ReadCommandOutput',
    TRANSACT_GET_COMMAND_OUTPUT_DATA_TYPE
];
export class DkTransactGetCommand extends DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = { dataType: TRANSACT_GET_COMMAND_INPUT_DATA_TYPE, hooks: TRANSACT_GET_COMMAND_INPUT_HOOK };
        this.outputMiddlewareConfig = { dataType: TRANSACT_GET_COMMAND_OUTPUT_DATA_TYPE, hooks: TRANSACT_GET_COMMAND_OUTPUT_HOOK };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = applyDefaults(this.input, defaults, ['returnConsumedCapacity']);
            const { data: postMiddlewareInput } = await executeMiddlewares([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: postDefaultsInput
            }, middleware);
            const { requests, ...rest } = postMiddlewareInput;
            const formattedInput = {
                transactItems: requests.map(request => ({ Get: upperCaseKeys(request) })),
                ...rest
            };
            const upperCaseInput = upperCaseKeys(formattedInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = lowerCaseKeys(output);
            const { responses, ...rest } = lowerCaseOutput;
            const items = responses
                ?.map(response => {
                return response.Item;
            })
                .filter((item) => !!item) || [];
            const formattedOutput = {
                ...rest,
                items
            };
            const { data: postMiddlewareOutput } = await executeMiddlewares([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: formattedOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity) {
                for (const consumedCapacity of postMiddlewareOutput.consumedCapacity) {
                    await executeMiddleware('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: consumedCapacity }, middleware);
                }
            }
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new TransactGetCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
