import { UpdateCommand } from '@aws-sdk/lib-dynamodb';
import { DkCommand } from './Command';
import { assertReturnValuesAttributes } from '../util/returnValuesAttributes';
import { lowerCaseKeys, upperCaseKeys } from '../util/keyCapitalize';
import { applyDefaults } from '../util/defaults';
import { executeMiddlewares, executeMiddleware } from '../Middleware';
const UPDATE_COMMAND_INPUT_DATA_TYPE = 'UpdateCommandInput';
const UPDATE_COMMAND_INPUT_HOOK = ['CommandInput', 'WriteCommandInput', UPDATE_COMMAND_INPUT_DATA_TYPE];
const UPDATE_COMMAND_OUTPUT_DATA_TYPE = 'UpdateCommandOutput';
const UPDATE_COMMAND_OUTPUT_HOOK = ['CommandOutput', 'WriteCommandOutput', UPDATE_COMMAND_OUTPUT_DATA_TYPE];
export class DkUpdateCommand extends DkCommand {
    constructor(input) {
        super(input);
        this.inputMiddlewareConfig = { dataType: UPDATE_COMMAND_INPUT_DATA_TYPE, hooks: UPDATE_COMMAND_INPUT_HOOK };
        this.outputMiddlewareConfig = { dataType: UPDATE_COMMAND_OUTPUT_DATA_TYPE, hooks: UPDATE_COMMAND_OUTPUT_HOOK };
        this.handleInput = async ({ defaults, middleware }) => {
            const postDefaultsInput = applyDefaults(this.input, defaults, [
                'returnConsumedCapacity',
                'returnItemCollectionMetrics',
                'returnValuesOnConditionCheckFailure'
            ]);
            const { data: postMiddlewareInput } = await executeMiddlewares([...this.inputMiddlewareConfig.hooks], {
                dataType: this.inputMiddlewareConfig.dataType,
                data: postDefaultsInput
            }, middleware);
            const upperCaseInput = upperCaseKeys(postMiddlewareInput);
            return upperCaseInput;
        };
        this.handleOutput = async (output, { middleware }) => {
            const lowerCaseOutput = lowerCaseKeys(output);
            const attributes = output.Attributes;
            assertReturnValuesAttributes(attributes, this.input.returnValues);
            const formattedOutput = {
                ...lowerCaseOutput,
                attributes
            };
            const { data: postMiddlewareOutput } = await executeMiddlewares([...this.outputMiddlewareConfig.hooks], {
                dataType: this.outputMiddlewareConfig.dataType,
                data: formattedOutput
            }, middleware);
            if (postMiddlewareOutput.consumedCapacity)
                await executeMiddleware('ConsumedCapacity', { dataType: 'ConsumedCapacity', data: postMiddlewareOutput.consumedCapacity }, middleware);
            if (postMiddlewareOutput.itemCollectionMetrics)
                await executeMiddleware('ItemCollectionMetrics', { dataType: 'ItemCollectionMetrics', data: postMiddlewareOutput.itemCollectionMetrics }, middleware);
            return postMiddlewareOutput;
        };
        this.send = async (clientConfig) => {
            const input = await this.handleInput(clientConfig);
            const output = await clientConfig.client.send(new UpdateCommand(input));
            return this.handleOutput(output, clientConfig);
        };
    }
}
