import { DkBatchWriteCommand } from '../command/BatchWrite';
export const batchWriteTableItems = async (Table, requests, input, dkClient = Table.dkClient) => {
    if (requests.length === 0) {
        return {
            unprocessedRequests: []
        };
    }
    const pageLimit = input?.pageLimit ? Math.min(input.pageLimit, 25) : 25;
    const recurse = async (remainingRequests) => {
        const currentRequests = remainingRequests.slice(0, pageLimit);
        const output = await dkClient.send(new DkBatchWriteCommand({
            ...input,
            requests: {
                [Table.tableName]: currentRequests
            }
        }));
        const unprocessedRequests = output.unprocessedRequests[Table.tableName] || [];
        const nextRemainingRequests = remainingRequests.slice(pageLimit);
        if (nextRemainingRequests.length === 0) {
            return {
                ...output,
                unprocessedRequests: unprocessedRequests
            };
        }
        const nextPage = await recurse(nextRemainingRequests);
        return {
            ...output,
            unprocessedRequests: [...unprocessedRequests, ...nextPage.unprocessedRequests]
        };
    };
    return recurse(requests);
};
export const batchWriteItems = async (KeySpace, requests, input) => batchWriteTableItems(KeySpace.Table, requests.map(request => {
    if ('put' in request) {
        return { put: KeySpace.withIndexKeys(request.put) };
    }
    return { delete: KeySpace.keyOf(request.delete) };
}), input, KeySpace.dkClient);
