import { DkQueryCommand } from '../command/Query';
export const queryTableItems = async (Table, input, dkClient = Table.dkClient) => {
    const recurse = async (totalCount, pageCursorKey) => {
        const { autoPage, pageLimit, totalLimit, ...inputRest } = input;
        const output = await dkClient.send(new DkQueryCommand({
            ...inputRest,
            tableName: Table.tableName,
            cursorKey: pageCursorKey,
            limit: pageLimit
        }));
        const { items, cursorKey, count = 0, ...rest } = output;
        const newTotalCount = totalCount + items.length;
        if (!autoPage || !cursorKey || (totalLimit && newTotalCount >= totalLimit)) {
            return {
                items: items.slice(0, totalLimit),
                cursorKey,
                count,
                ...rest
            };
        }
        const nextPage = await recurse(newTotalCount, cursorKey);
        return {
            items: [...items, ...nextPage.items].slice(0, totalLimit),
            cursorKey: nextPage.cursorKey,
            count: count + (nextPage.count || 0),
            ...rest
        };
    };
    return recurse(0, input?.cursorKey);
};
export const queryItems = async (KeySpace, input) => {
    const output = await queryTableItems(KeySpace.Table, input, KeySpace.dkClient);
    return {
        ...output,
        items: output.items.map(item => {
            KeySpace.assertAttributesAndKeys(item);
            return KeySpace.omitIndexKeys(item);
        })
    };
};
