import { primaryIndex } from '../Table';
import { queryTableItems } from './query';
import { createQueryQuickSort } from '../util/createSortKeyQuery';
export const queryQuickTableItems = async (Table, input, dkClient = Table.dkClient) => {
    const index = input.index || primaryIndex;
    const { hashKeyParams, beginsWith, greaterThan, lessThan, ...inputRest } = input;
    const hashKey = Table.config.indexes[index].hash.key;
    const hashValue = hashKeyParams[hashKey];
    const sortKey = Table.config.indexes[index].sort?.key;
    const sortParams = createQueryQuickSort(sortKey, { beginsWith, greaterThan, lessThan });
    const output = await queryTableItems(Table, {
        ...inputRest,
        keyConditionExpression: `${hashKey} = :hashValue ${sortParams.keyConditionExpression || ''}`,
        expressionAttributeValues: {
            [`:hashValue`]: hashValue,
            ...sortParams.expressionAttributeValues,
            ...input.expressionAttributeValues
        }
    }, dkClient);
    return output;
};
export const queryQuickItems = async (KeySpace, input) => {
    const index = input.index || primaryIndex;
    const hashKeyValue = KeySpace.indexAttributeValue(index, KeySpace.Table.config.indexes[index].hash.key, input.hashKeyParams);
    const hashKeyParams = {
        [KeySpace.Table.config.indexes[index].hash.key]: hashKeyValue
    };
    const output = await queryQuickTableItems(KeySpace.Table, { ...input, hashKeyParams }, KeySpace.dkClient);
    return {
        ...output,
        items: output.items.map(item => {
            KeySpace.assertAttributesAndKeys(item);
            return KeySpace.omitIndexKeys(item);
        })
    };
};
