import { DkScanCommand } from '../command/Scan';
export const scanTableItems = async (Table, input) => {
    const recurse = async (totalCount, pageCursorKey) => {
        const { autoPage, pageLimit, totalLimit, ...inputRest } = input || {};
        const output = await Table.dkClient.send(new DkScanCommand({
            ...inputRest,
            tableName: Table.tableName,
            cursorKey: pageCursorKey,
            limit: pageLimit
        }));
        const { items, cursorKey, count = 0, ...rest } = output;
        const newTotalCount = totalCount + items.length;
        if (!autoPage || !cursorKey || (totalLimit && newTotalCount >= totalLimit)) {
            return {
                items: items.slice(0, totalLimit),
                cursorKey,
                count,
                ...rest
            };
        }
        const nextPage = await recurse(newTotalCount, cursorKey);
        return {
            items: [...items, ...nextPage.items].slice(0, totalLimit),
            cursorKey: nextPage.cursorKey,
            count: count + (nextPage.count || 0),
            ...rest
        };
    };
    return recurse(0, input?.cursorKey);
};
