export const dkSetAttributeOnWriteMiddleware = (key, setter) => {
    const batchWriteHandler = {
        hook: 'BatchWriteCommandInput',
        handler: ({ data: batchWriteCommandInput }) => {
            return {
                ...batchWriteCommandInput,
                requests: Object.fromEntries(Object.entries(batchWriteCommandInput.requests).map(([tableName, requests]) => [
                    tableName,
                    requests.map(request => {
                        if ('put' in request) {
                            return {
                                put: {
                                    ...request.put,
                                    [key]: setter()
                                }
                            };
                        }
                        return request;
                    })
                ]))
            };
        }
    };
    const putHandler = {
        hook: 'PutCommandInput',
        handler: ({ data: putCommandInput }) => {
            return {
                ...putCommandInput,
                item: {
                    ...putCommandInput.item,
                    [key]: setter()
                }
            };
        }
    };
    const transactWriteHandler = {
        hook: 'TransactWriteCommandInput',
        handler: ({ data: transactWriteCommandInput }) => {
            return {
                ...transactWriteCommandInput,
                requests: transactWriteCommandInput.requests.map(request => {
                    if (request.type === 'put') {
                        return {
                            ...request,
                            item: {
                                ...request.item,
                                [key]: setter()
                            }
                        };
                    }
                    if (request.type === 'update') {
                        if (!request.updateExpression.startsWith('SET'))
                            return request;
                        return {
                            ...request,
                            updateExpression: request.updateExpression + `, ${key} = :${key}`,
                            expressionAttributeValues: {
                                ...request.expressionAttributeValues,
                                [`:${key}`]: setter()
                            }
                        };
                    }
                    return request;
                })
            };
        }
    };
    const updateHandler = {
        hook: 'UpdateCommandInput',
        handler: ({ data: updateCommandInput }) => {
            if (!updateCommandInput.updateExpression.startsWith('SET'))
                return;
            return {
                ...updateCommandInput,
                updateExpression: updateCommandInput.updateExpression + `, ${key} = :${key}`,
                expressionAttributeValues: {
                    ...updateCommandInput.expressionAttributeValues,
                    [`:${key}`]: setter()
                }
            };
        }
    };
    return [batchWriteHandler, putHandler, transactWriteHandler, updateHandler];
};
