import { PrimaryIndex, Table } from './Table';
import { GenericAttributes, Remap } from './util/utils';
import { DynamoDBDocumentClient } from '@aws-sdk/lib-dynamodb';
import { U } from 'ts-toolbelt';
import { DkClient, DkClientConfig } from './Client';
export declare namespace KeySpace {
    type GetKeyParams<K extends KeySpace, Index extends K['indexes'][number]> = Remap<Exclude<GetIndexKeyValueParamsMap<K>[Index], undefined | never>>;
    type GetIndexAttributeKeys<K extends KeySpace, Index extends K['indexes'][number] = K['indexes'][number]> = Table.GetIndexAttributeKeyMap<K['Table']>[Index];
    type GetIndexKeyMap<K extends KeySpace> = {
        [x in K['indexes'][number]]: Table.GetIndexKeyMap<K['Table']>[x];
    };
    type GetIndexKey<K extends KeySpace, Index extends K['indexes'][number]> = Remap<U.IntersectOf<KeySpace.GetIndexKeyMap<K>[Index]>>;
    type GetAttributes<K extends KeySpace> = Parameters<K['indexKeysOf']>[0];
    type GetAttributesAndKeys<K extends KeySpace> = GetAttributes<K> & KeySpace.GetIndexKey<K, K['indexes'][number]>;
    type GetIndexValueParamsMap<K extends KeySpace> = {
        [x in K['indexes'][number]]: {
            [y in keyof K['indexValueHandlers'][x]]: Parameters<K['indexValueHandlers'][x][y]>[0];
        };
    };
    type GetIndexKeyValueParamsMap<K extends KeySpace> = {
        [x in K['indexes'][number]]: U.IntersectOf<Exclude<GetIndexValueParamsMap<K>[x][keyof GetIndexValueParamsMap<K>[x]], undefined>>;
    };
    type GetIndexKeyValueParams<K extends KeySpace, Index extends K['indexes'][number]> = Remap<GetIndexKeyValueParamsMap<K>[Index]> extends never ? undefined : Remap<GetIndexKeyValueParamsMap<K>[Index]>;
    type GetIndexHashKeyValueParamsMap<K extends KeySpace> = {
        [x in K['indexes'][number]]: GetIndexValueParamsMap<K>[x][Table.GetIndexHashKey<K['Table'], x>];
    };
}
export type IndexValueHandlersType<ParentTable extends Table = Table, Attributes extends GenericAttributes = GenericAttributes, SecondaryIndex extends ParentTable['secondaryIndexes'][number] = never> = {
    [x in string & Exclude<PrimaryIndex | SecondaryIndex, never>]: {
        [y in keyof Table.GetIndexKey<ParentTable, x>]: (params: Attributes) => Table.GetIndexKey<ParentTable, x>[y];
    };
};
export interface KeySpaceConfig<ParentTable extends Table = Table, Attributes extends GenericAttributes = GenericAttributes, SecondaryIndex extends ParentTable['secondaryIndexes'][number] = never, IndexValueHandlers extends IndexValueHandlersType<ParentTable, Attributes, SecondaryIndex> = any> extends Partial<DkClientConfig> {
    indexValueHandlers: IndexValueHandlers;
}
export declare class KeySpace<ParentTable extends Table = any, Attributes extends GenericAttributes = GenericAttributes, SecondaryIndex extends ParentTable['secondaryIndexes'][number] = any, IndexValueHandlers extends IndexValueHandlersType<ParentTable, Attributes, SecondaryIndex> = any> {
    Table: ParentTable;
    config: KeySpaceConfig<ParentTable, Attributes, SecondaryIndex, IndexValueHandlers>;
    client: DynamoDBDocumentClient;
    dkClient: DkClient;
    tableName: string;
    indexValueHandlers: IndexValueHandlers;
    constructor(Table: ParentTable, config: KeySpaceConfig<ParentTable, Attributes, SecondaryIndex, IndexValueHandlers>);
    configure<ConfigIndexValueHandlers extends IndexValueHandlersType<ParentTable, Attributes, SecondaryIndex> = any>(config: KeySpaceConfig<ParentTable, Attributes, SecondaryIndex, ConfigIndexValueHandlers>): KeySpace<ParentTable, Attributes, SecondaryIndex, ConfigIndexValueHandlers>;
    get indexes(): Array<Exclude<this['Table']['primaryIndex'] | SecondaryIndex, never>>;
    get primaryIndex(): this['Table']['primaryIndex'];
    get secondaryIndexes(): Array<SecondaryIndex>;
    indexAttributeKeys: <Index extends this["indexes"][number]>(index: Index) => Table.GetIndexAttributeKeys<this["Table"], Index>[];
    get attributeKeys(): Array<KeySpace.GetIndexAttributeKeys<this, this['indexes'][number]>>;
    indexAttributeValue<Index extends this['indexes'][number], Key extends keyof this['config']['indexValueHandlers'][Index]>(index: Index, key: Key, params: Parameters<this['config']['indexValueHandlers'][Index][Key]>[0]): ReturnType<this['config']['indexValueHandlers'][Index][Key]>;
    keyOf(params: KeySpace.GetIndexKeyValueParams<this, this['primaryIndex']>): KeySpace.GetIndexKey<this, this['primaryIndex']>;
    indexKeyOf<Index extends this['indexes'][number]>(index: Index, params: KeySpace.GetIndexKeyValueParams<this, Index>): KeySpace.GetIndexKey<this, Index>;
    indexKeysOf(item: Attributes): KeySpace.GetIndexKey<this, this['indexes'][number]>;
    withIndexKeys(item: Attributes): KeySpace.GetAttributesAndKeys<this>;
    omitIndexKeys(item: KeySpace.GetAttributesAndKeys<this>): Attributes;
    assertAttributesAndKeys: (item: GenericAttributes) => asserts item is KeySpace.GetAttributesAndKeys<this>;
}
