import { DynamoDBDocumentClient } from '@aws-sdk/lib-dynamodb';
import { U, A } from 'ts-toolbelt';
import { DkClient, DkClientConfig } from './Client';
import { GenericAttributes, Remap } from './util/utils';
import { KeySpace } from './KeySpace';
export declare const primaryIndex: "primaryIndex";
export type PrimaryIndex = typeof primaryIndex;
export declare namespace Table {
    type GetIndexAttributeKeyMap<T extends Table> = {
        [x in T['indexes'][number]]: T['config']['indexes'][x]['hash']['key'] | Exclude<T['config']['indexes'][x]['sort'] extends IndexAttributeConfig ? T['config']['indexes'][x]['sort']['key'] : never, never>;
    };
    type GetIndexAttributeKeys<T extends Table, Index extends T['indexes'][number] = T['indexes'][number]> = Table.GetIndexAttributeKeyMap<T>[Index];
    type GetIndexHashKeyMap<T extends Table> = {
        [x in T['indexes'][number]]: T['config']['indexes'][x]['hash']['key'];
    };
    type GetIndexHashKey<T extends Table, Index extends T['indexes'][number] = T['indexes'][number]> = Table.GetIndexHashKeyMap<T>[Index];
    type GetIndexKeyMap<T extends Table> = {
        [x in T['indexes'][number]]: {
            [y in T['config']['indexes'][x]['hash']['key']]: IndexAttributeValueToType<T['config']['indexes'][x]['hash']['value']>;
        } & (T['config']['indexes'][x]['sort'] extends IndexAttributeConfig ? {
            [y in T['config']['indexes'][x]['sort']['key']]: IndexAttributeValueToType<T['config']['indexes'][x]['sort']['value']>;
        } : {});
    };
    type GetIndexKey<T extends Table, Index extends T['indexes'][number] | never | undefined = T['indexes'][number]> = Remap<U.IntersectOf<Table.GetIndexKeyMap<T>[Exclude<Index, never | undefined>]>>;
    type GetIndexCursorKey<T extends Table, Index extends T['secondaryIndexes'][number] | never | undefined> = Remap<A.Cast<Table.GetIndexKey<T, T['primaryIndex'] | Index>, Table.GetIndexKey<T, T['primaryIndex']>>>;
    type GetAttributes<T extends Table> = Remap<T['secondaryIndexes'][number] extends string ? Table.GetIndexKeyMap<T>[T['primaryIndex']] & Partial<U.IntersectOf<Table.GetIndexKeyMap<T>[T['secondaryIndexes'][number]]>> : Table.GetIndexKeyMap<T>[T['primaryIndex']]>;
}
export type IndexAttributeValue = 'string' | 'string?' | 'number' | 'number?';
export type IndexAttributeValueToType<T extends IndexAttributeValue = 'string'> = T extends 'string' ? string : T extends 'string?' ? string | undefined : T extends 'number' ? number : T extends 'number?' ? number | undefined : never;
export interface IndexAttributeConfig<Key extends string = string, Value extends IndexAttributeValue = IndexAttributeValue> {
    key: Key;
    value: Value;
}
export interface PrimaryIndexConfig<HashKey extends string = string, HashValue extends IndexAttributeValue = IndexAttributeValue, SortKey extends string = string, SortValue extends IndexAttributeValue = IndexAttributeValue> {
    hash: IndexAttributeConfig<HashKey, Extract<HashValue, 'string' | 'number'>>;
    sort?: IndexAttributeConfig<SortKey, SortValue>;
}
export type IndexProjection<Attributes extends string> = undefined | never[] | Attributes[];
export interface SecondaryIndexConfig<HashKey extends string = string, HashValue extends IndexAttributeValue = IndexAttributeValue, SortKey extends string = string, SortValue extends IndexAttributeValue = IndexAttributeValue, ProjectionKeys extends string = string, Projection extends IndexProjection<ProjectionKeys> = IndexProjection<ProjectionKeys>> extends PrimaryIndexConfig<HashKey, HashValue, SortKey, SortValue> {
    projection?: Projection;
}
export interface TableConfig<AttributeKey extends string = string, AttributeValue extends IndexAttributeValue = IndexAttributeValue, ProjectionKeys extends string = string, Projection extends IndexProjection<ProjectionKeys> = IndexProjection<ProjectionKeys>> extends Partial<DkClientConfig> {
    client: DynamoDBDocumentClient;
    name: string;
    indexes: Record<PrimaryIndex, PrimaryIndexConfig<AttributeKey, AttributeValue, AttributeKey, AttributeValue>> & Record<Exclude<string, PrimaryIndex>, SecondaryIndexConfig<AttributeKey, AttributeValue, AttributeKey, AttributeValue, ProjectionKeys, Projection>>;
}
export declare class Table<AttributeKey extends string = any, AttributeValue extends IndexAttributeValue = any, ProjectionKeys extends string = any, Projection extends IndexProjection<ProjectionKeys> = any, Config extends TableConfig<AttributeKey, AttributeValue, ProjectionKeys, Projection> = TableConfig<AttributeKey, AttributeValue, ProjectionKeys, Projection>> {
    config: Config;
    client: DynamoDBDocumentClient;
    dkClient: DkClient;
    tableName: string;
    constructor(config: Config);
    configure<ConfigAttributeKey extends string, ConfigAttributeValue extends IndexAttributeValue, ConfigProjectionKeys extends string, ConfigProjection extends IndexProjection<ConfigProjectionKeys>, ConfigConfig extends TableConfig<ConfigAttributeKey, ConfigAttributeValue, ConfigProjectionKeys, ConfigProjection>>(config: ConfigConfig): Table<ConfigAttributeKey, ConfigAttributeValue, ConfigProjectionKeys, ConfigProjection, ConfigConfig>;
    get KeySpace(): {
        new <Attributes extends GenericAttributes = GenericAttributes, SecondaryIndex extends string & Exclude<string & keyof Config["indexes"], "primaryIndex"> = never>(): {
            client: DynamoDBDocumentClient;
            dkClient: DkClient;
            tableName: string;
            indexValueHandlers: any;
            Table: Table<AttributeKey, AttributeValue, ProjectionKeys, Projection, Config>;
            config: import("./KeySpace").KeySpaceConfig<Table<AttributeKey, AttributeValue, ProjectionKeys, Projection, Config>, Attributes, SecondaryIndex, any>;
            configure<ConfigIndexValueHandlers extends import("./KeySpace").IndexValueHandlersType<Table<AttributeKey, AttributeValue, ProjectionKeys, Projection, Config>, Attributes, SecondaryIndex> = any>(config: import("./KeySpace").KeySpaceConfig<Table<AttributeKey, AttributeValue, ProjectionKeys, Projection, Config>, Attributes, SecondaryIndex, ConfigIndexValueHandlers>): KeySpace<Table<AttributeKey, AttributeValue, ProjectionKeys, Projection, Config>, Attributes, SecondaryIndex, ConfigIndexValueHandlers>;
            readonly indexes: ("primaryIndex" | Exclude<SecondaryIndex, never>)[];
            readonly primaryIndex: "primaryIndex";
            readonly secondaryIndexes: SecondaryIndex[];
            indexAttributeKeys: <Index extends "primaryIndex" | Exclude<SecondaryIndex, never>>(index: Index) => Table.GetIndexAttributeKeys<Table<AttributeKey, AttributeValue, ProjectionKeys, Projection, Config>, Index>[];
            readonly attributeKeys: KeySpace.GetIndexAttributeKeys<any, "primaryIndex" | Exclude<SecondaryIndex, never>>[];
            indexAttributeValue<Index_1 extends "primaryIndex" | Exclude<SecondaryIndex, never>, Key extends string | number | symbol>(index: Index_1, key: Key, params: unknown): any;
            keyOf(params: KeySpace.GetIndexKeyValueParams<any, "primaryIndex">): Remap<U.IntersectOf<KeySpace.GetIndexKeyMap<any>["primaryIndex"]>>;
            indexKeyOf<Index_2 extends "primaryIndex" | Exclude<SecondaryIndex, never>>(index: Index_2, params: KeySpace.GetIndexKeyValueParams<any, Index_2>): Remap<U.IntersectOf<KeySpace.GetIndexKeyMap<any>[Index_2]>>;
            indexKeysOf(item: Attributes): Remap<U.IntersectOf<KeySpace.GetIndexKeyMap<any>["primaryIndex" | Exclude<SecondaryIndex, never>]>>;
            withIndexKeys(item: Attributes): KeySpace.GetAttributesAndKeys<any>;
            omitIndexKeys(item: KeySpace.GetAttributesAndKeys<any>): Attributes;
            assertAttributesAndKeys: (item: GenericAttributes) => asserts item is KeySpace.GetAttributesAndKeys<any>;
        };
    };
    get indexes(): Array<string & keyof Config['indexes']>;
    get primaryIndex(): PrimaryIndex;
    get secondaryIndexes(): Array<string & Exclude<this['indexes'][number], this['primaryIndex']>>;
    indexAttributeKeys: <Index extends this["indexes"][number]>(index: Index) => Table.GetIndexAttributeKeys<this, Index>[];
    get attributeKeys(): Array<Table.GetIndexAttributeKeys<this, this['indexes'][number]>>;
    pickPrimaryIndexKey: <T extends Partial<Remap<this["secondaryIndexes"][number] extends string ? Table.GetIndexKeyMap<this>[this["primaryIndex"]] & Partial<U.IntersectOf<Table.GetIndexKeyMap<this>[this["secondaryIndexes"][number]]>> : Table.GetIndexKeyMap<this>[this["primaryIndex"]]>>>(object: T) => Table.GetIndexKey<this, this['primaryIndex']>;
    omitIndexKeys<T extends Partial<Table.GetAttributes<this>>>(object: T): Omit<T, AttributeKey>;
}
