import { BatchGetCommandInput, BatchGetCommandOutput } from '@aws-sdk/lib-dynamodb';
import { DkCommand } from './Command';
import { LowerCaseObjectKeys } from '../util/keyCapitalize';
import { DkClientConfig } from '../Client';
import { KeysAndAttributes } from '@aws-sdk/client-dynamodb';
import { GenericAttributes } from '../util/utils';
declare const BATCH_GET_COMMAND_INPUT_DATA_TYPE: "BatchGetCommandInput";
declare const BATCH_GET_COMMAND_INPUT_HOOK: readonly ["CommandInput", "ReadCommandInput", "BatchGetCommandInput"];
declare const BATCH_GET_COMMAND_OUTPUT_DATA_TYPE: "BatchGetCommandOutput";
declare const BATCH_GET_COMMAND_OUTPUT_HOOK: readonly ["CommandOutput", "ReadCommandOutput", "BatchGetCommandOutput"];
export interface DkBatchGetCommandInput<Key extends GenericAttributes = GenericAttributes> extends LowerCaseObjectKeys<Omit<BatchGetCommandInput, 'RequestItems'>> {
    requests: Record<string, LowerCaseObjectKeys<Omit<KeysAndAttributes, 'Keys' | 'AttributesToGet'>> & {
        keys: Key[];
    }>;
}
export interface DkBatchGetCommandOutput<Attributes extends GenericAttributes = GenericAttributes, Key extends GenericAttributes = GenericAttributes> extends LowerCaseObjectKeys<Omit<BatchGetCommandOutput, 'Responses' | 'UnprocessedKeys'>> {
    items: Record<string, Attributes[]>;
    unprocessedRequests: Record<string, (LowerCaseObjectKeys<Omit<KeysAndAttributes, 'Keys' | 'AttributesToGet'>> & {
        keys: Key[];
    }) | undefined>;
}
export declare class DkBatchGetCommand<Attributes extends GenericAttributes = GenericAttributes, Key extends GenericAttributes = GenericAttributes> extends DkCommand<typeof BATCH_GET_COMMAND_INPUT_DATA_TYPE, (typeof BATCH_GET_COMMAND_INPUT_HOOK)[number], DkBatchGetCommandInput<Key>, BatchGetCommandInput, typeof BATCH_GET_COMMAND_OUTPUT_DATA_TYPE, (typeof BATCH_GET_COMMAND_OUTPUT_HOOK)[number], DkBatchGetCommandOutput<Attributes, Key>, BatchGetCommandOutput> {
    constructor(input: DkBatchGetCommandInput<Key>);
    inputMiddlewareConfig: {
        dataType: "BatchGetCommandInput";
        hooks: readonly ["CommandInput", "ReadCommandInput", "BatchGetCommandInput"];
    };
    outputMiddlewareConfig: {
        dataType: "BatchGetCommandOutput";
        hooks: readonly ["CommandOutput", "ReadCommandOutput", "BatchGetCommandOutput"];
    };
    handleInput: ({ defaults, middleware }: DkClientConfig) => Promise<BatchGetCommandInput>;
    handleOutput: (output: BatchGetCommandOutput, { middleware }: DkClientConfig) => Promise<DkBatchGetCommandOutput<Attributes, Key>>;
    send: (clientConfig: DkClientConfig) => Promise<DkBatchGetCommandOutput<Attributes, Key>>;
}
export {};
