import { BatchWriteCommandInput, BatchWriteCommandOutput } from '@aws-sdk/lib-dynamodb';
import { DkCommand } from './Command';
import { GenericAttributes } from '../util/utils';
import { LowerCaseObjectKeys } from '../util/keyCapitalize';
import { DkClientConfig } from '../Client';
declare const BATCH_WRITE_COMMAND_INPUT_DATA_TYPE: "BatchWriteCommandInput";
declare const BATCH_WRITE_COMMAND_INPUT_HOOK: readonly ["CommandInput", "WriteCommandInput", "BatchWriteCommandInput"];
declare const BATCH_WRITE_COMMAND_OUTPUT_DATA_TYPE: "BatchWriteCommandOutput";
declare const BATCH_WRITE_COMMAND_OUTPUT_HOOK: readonly ["CommandOutput", "WriteCommandOutput", "BatchWriteCommandOutput"];
export interface DkBatchWriteCommandInput<Attributes extends GenericAttributes = GenericAttributes, Key extends GenericAttributes = GenericAttributes> extends LowerCaseObjectKeys<Omit<BatchWriteCommandInput, 'RequestItems'>> {
    requests: Record<string, Array<{
        put: Attributes;
    } | {
        delete: Key;
    }>>;
}
export interface DkBatchWriteCommandOutput<Attributes extends GenericAttributes = GenericAttributes, Key extends GenericAttributes = GenericAttributes> extends LowerCaseObjectKeys<Omit<BatchWriteCommandOutput, 'UnprocessedItems'>> {
    unprocessedRequests: Record<string, Array<{
        put: Attributes;
    } | {
        delete: Key;
    }>>;
}
export declare class DkBatchWriteCommand<Attributes extends GenericAttributes = GenericAttributes, Key extends GenericAttributes = GenericAttributes> extends DkCommand<typeof BATCH_WRITE_COMMAND_INPUT_DATA_TYPE, (typeof BATCH_WRITE_COMMAND_INPUT_HOOK)[number], DkBatchWriteCommandInput<Attributes, Key>, BatchWriteCommandInput, typeof BATCH_WRITE_COMMAND_OUTPUT_DATA_TYPE, (typeof BATCH_WRITE_COMMAND_OUTPUT_HOOK)[number], DkBatchWriteCommandOutput<Attributes, Key>, BatchWriteCommandOutput> {
    constructor(input: DkBatchWriteCommandInput<Attributes, Key>);
    inputMiddlewareConfig: {
        dataType: "BatchWriteCommandInput";
        hooks: readonly ["CommandInput", "WriteCommandInput", "BatchWriteCommandInput"];
    };
    outputMiddlewareConfig: {
        dataType: "BatchWriteCommandOutput";
        hooks: readonly ["CommandOutput", "WriteCommandOutput", "BatchWriteCommandOutput"];
    };
    handleInput: ({ defaults, middleware }: DkClientConfig) => Promise<BatchWriteCommandInput>;
    handleOutput: (output: BatchWriteCommandOutput, { middleware }: DkClientConfig) => Promise<DkBatchWriteCommandOutput<Attributes, Key>>;
    send: (clientConfig: DkClientConfig) => Promise<DkBatchWriteCommandOutput<Attributes, Key>>;
}
export {};
