import { QueryCommandInput, QueryCommandOutput } from '@aws-sdk/lib-dynamodb';
import { DkCommand } from './Command';
import { GenericAttributes } from '../util/utils';
import { LowerCaseObjectKeys } from '../util/keyCapitalize';
import { DkClientConfig } from '../Client';
export declare enum DkQueryItemsSort {
    ASCENDING = "ascending",
    DESCENDING = "descending"
}
declare const QUERY_COMMAND_INPUT_DATA_TYPE: "QueryCommandInput";
declare const QUERY_COMMAND_INPUT_HOOK: readonly ["CommandInput", "ReadCommandInput", "QueryCommandInput"];
declare const QUERY_COMMAND_OUTPUT_DATA_TYPE: "QueryCommandOutput";
declare const QUERY_COMMAND_OUTPUT_HOOK: readonly ["CommandOutput", "ReadCommandOutput", "QueryCommandOutput"];
export interface DkQueryCommandInput<CursorKey extends GenericAttributes = GenericAttributes> extends LowerCaseObjectKeys<Omit<QueryCommandInput, 'ExclusiveStartKey' | 'IndexName' | 'ScanIndexForward' | 'AttributesToGet' | 'ConditionalOperator'>> {
    index?: string | undefined;
    cursorKey?: CursorKey;
    sort?: DkQueryItemsSort;
}
export interface DkQueryCommandOutput<Attributes extends GenericAttributes = GenericAttributes, CursorKey extends GenericAttributes = GenericAttributes> extends LowerCaseObjectKeys<Omit<QueryCommandOutput, 'Items' | 'LastEvaluatedKey'>> {
    items: Array<Attributes>;
    cursorKey?: CursorKey;
}
export declare class DkQueryCommand<Attributes extends GenericAttributes = GenericAttributes, CursorKey extends GenericAttributes = GenericAttributes> extends DkCommand<typeof QUERY_COMMAND_INPUT_DATA_TYPE, (typeof QUERY_COMMAND_INPUT_HOOK)[number], DkQueryCommandInput<CursorKey>, QueryCommandInput, typeof QUERY_COMMAND_OUTPUT_DATA_TYPE, (typeof QUERY_COMMAND_OUTPUT_HOOK)[number], DkQueryCommandOutput<Attributes, CursorKey>, QueryCommandOutput> {
    constructor(input: DkQueryCommandInput<CursorKey>);
    inputMiddlewareConfig: {
        dataType: "QueryCommandInput";
        hooks: readonly ["CommandInput", "ReadCommandInput", "QueryCommandInput"];
    };
    outputMiddlewareConfig: {
        dataType: "QueryCommandOutput";
        hooks: readonly ["CommandOutput", "ReadCommandOutput", "QueryCommandOutput"];
    };
    handleInput: ({ defaults, middleware }: DkClientConfig) => Promise<QueryCommandInput>;
    handleOutput: (output: QueryCommandOutput, { middleware }: DkClientConfig) => Promise<DkQueryCommandOutput<Attributes, CursorKey>>;
    send: (clientConfig: DkClientConfig) => Promise<DkQueryCommandOutput<Attributes, CursorKey>>;
}
export {};
