import { TransactWriteCommandInput, TransactWriteCommandOutput } from '@aws-sdk/lib-dynamodb';
import { DkCommand } from './Command';
import { GenericAttributes } from '../util/utils';
import { LowerCaseObjectKeys } from '../util/keyCapitalize';
import { DkClientConfig } from '../Client';
import { ConditionCheck, Delete, Put, Update } from '@aws-sdk/client-dynamodb';
declare const TRANSACT_WRITE_COMMAND_INPUT_DATA_TYPE: "TransactWriteCommandInput";
declare const TRANSACT_WRITE_COMMAND_INPUT_HOOK: readonly ["CommandInput", "WriteCommandInput", "TransactWriteCommandInput"];
declare const TRANSACT_WRITE_COMMAND_OUTPUT_DATA_TYPE: "TransactWriteCommandOutput";
declare const TRANSACT_WRITE_COMMAND_OUTPUT_HOOK: readonly ["CommandOutput", "WriteCommandOutput", "TransactWriteCommandOutput"];
export interface DkTransactWriteCommandInputConditionCheck<Key extends GenericAttributes | undefined = GenericAttributes> extends LowerCaseObjectKeys<Omit<ConditionCheck, 'Key'>> {
    type: 'conditionCheck';
    key: Key;
}
export interface DkTransactWriteCommandInputPut<Attributes extends GenericAttributes = GenericAttributes> extends LowerCaseObjectKeys<Omit<Put, 'Item'>> {
    type: 'put';
    item: Attributes;
}
export interface DkTransactWriteCommandInputDelete<Key extends GenericAttributes | undefined = GenericAttributes> extends LowerCaseObjectKeys<Omit<Delete, 'Key'>> {
    type: 'delete';
    key: Key;
}
export interface DkTransactWriteCommandInputUpdate<Key extends GenericAttributes | undefined = GenericAttributes> extends LowerCaseObjectKeys<Omit<Update, 'Key'>> {
    type: 'update';
    key: Key;
}
export interface DkTransactWriteCommandInput<Attributes extends GenericAttributes = GenericAttributes, Key extends GenericAttributes = GenericAttributes> extends LowerCaseObjectKeys<Omit<TransactWriteCommandInput, 'TransactItems'>> {
    requests: Array<DkTransactWriteCommandInputConditionCheck<Key> | DkTransactWriteCommandInputPut<Attributes> | DkTransactWriteCommandInputDelete<Key> | DkTransactWriteCommandInputUpdate<Key>>;
}
export interface DkTransactWriteCommandOutput extends LowerCaseObjectKeys<TransactWriteCommandOutput> {
}
export declare class DkTransactWriteCommand<Attributes extends GenericAttributes = GenericAttributes, Key extends GenericAttributes = GenericAttributes> extends DkCommand<typeof TRANSACT_WRITE_COMMAND_INPUT_DATA_TYPE, (typeof TRANSACT_WRITE_COMMAND_INPUT_HOOK)[number], DkTransactWriteCommandInput<Attributes, Key>, TransactWriteCommandInput, typeof TRANSACT_WRITE_COMMAND_OUTPUT_DATA_TYPE, (typeof TRANSACT_WRITE_COMMAND_OUTPUT_HOOK)[number], DkTransactWriteCommandOutput, TransactWriteCommandOutput> {
    constructor(input: DkTransactWriteCommandInput<Attributes, Key>);
    inputMiddlewareConfig: {
        dataType: "TransactWriteCommandInput";
        hooks: readonly ["CommandInput", "WriteCommandInput", "TransactWriteCommandInput"];
    };
    outputMiddlewareConfig: {
        dataType: "TransactWriteCommandOutput";
        hooks: readonly ["CommandOutput", "WriteCommandOutput", "TransactWriteCommandOutput"];
    };
    handleInput: ({ defaults, middleware }: DkClientConfig) => Promise<TransactWriteCommandInput>;
    handleOutput: (output: TransactWriteCommandOutput, { middleware }: DkClientConfig) => Promise<DkTransactWriteCommandOutput>;
    send: (clientConfig: DkClientConfig) => Promise<DkTransactWriteCommandOutput>;
}
export {};
